# This file is part of M4OPT
# Copyright (C) 2011-2015 Sergey Poznyakoff
#
# M4OPT is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# M4OPT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with M4OPT.  If not, see <http://www.gnu.org/licenses/>.

m4_include(composite.m4)

# m4_argc(...)
# ------------
# Expands to the number of arguments supplied.
#
m4_define(`m4_argc',`$#')

# m4_ifempty(LIST, IF-TRUE [, IF-FALSE])
# --------------------------------------
# Expands to IF-TRUE if LIST is not empty, otherwise to IF-FALSE.
#
m4_define(`m4_ifempty', `m4_ifelse(`$1',`',`$2',`$3')')

# m4_quote_names(LIST)
# --------------------
# Produce a quoted comma-delimited list of doubly-quoted names from LIST.
# The result is suitable for use as second argument to m4_foreach, so that
# none of its elements are expanded while processing.
m4_define(`m4_quote_names', `m4_quote(m4_dquote(m4_dquote_elt($@)))')

# m4opt_list_if_member(LIST, ELT, IF-TRUE [, IF-FALSE])
# -----------------------------------------------------
# Expands to IF-TRUE if ELT is contained in the comma-delimited LIST of
# elements.  Otherwise, expands to IF-FALSE.
m4_define(`m4opt_list_if_member',`_m4opt_list_if_member__(`$3',`$4',`$2',$1)')
# _m4opt_list_if_member(IF-TRUE, IF-FALSE, ELT, ...)
# --------------------------------------------------
# Auxiliary function for m4opt_list_if_member.  Recursively scans the list
# of arguments beginning with $4 and stops when ELT is found, or end of
# the list is reached.  Expands to IF-TRUE or IF-FALSE, depending on
# whether ELT is found.
#
m4_define(`_m4opt_list_if_member__',
`m4_ifelse(`$4',`',`$2',
`$3',`$4',`$1',`_m4opt_list_if_member__(`$1',`$2',`$3',m4_shift(m4_shift(m4_shift(m4_shift($@)))))')')

# m4opt_list_remove(LIST, ELT)
# ----------------------------
# Removes ELT from the comma-delimited LIST.
#
m4_define(`m4opt_list_remove',
`m4_pushdef(`__L__',m4_dquote($1))m4_dnl
m4_popdef(`$1')m4_dnl
m4_pushdef(`$1',m4_quote(m4_shift(m4_dquote_elt(m4opt_list_remove_(`$2',__L__)))))m4_dnl
m4_popdef(`__L__')')

# m4opt_list_remove_(elt, ...)
m4_define(`m4opt_list_remove_',
`m4_ifelse(`$#',1,`',
`$#',2,`m4_ifelse(`$1',`$2',,`,m4_quote(`$2')')',
`m4_ifelse(`$1',`$2',,`,m4_quote(`$2')')m4opt_list_remove_(`$1',m4_shift(m4_shift($@)))')')

# m4opt_mangle_option(NAME)
# -------------------------
# Converts NAME to a option name by prefixing it with double-dash.
m4_define(`m4opt_mangle_option',`--`$1'')

# _m4opt_list - a list of defined option names
# --------------------------------------------
#
m4_define(`_m4opt_list')

# M4OPT_SET_OPTION(NAME [, VALUE])
# --------------------------------
# Sets option NAME to VALUE
m4_define(`M4OPT_SET_OPTION',
`m4_pushdef(`__opt_name__',m4_dquote(m4opt_mangle_option(`$1')))m4_dnl
m4opt_list_if_member(`_m4opt_list',`$1',`m4_popdef(__opt_name__)',
`m4_define(`_m4opt_list',m4_ifempty(m4_quote(_m4opt_list),
m4_dquote(m4_dquote(`$1')),`m4_quote(m4_dquote(_m4opt_list), m4_dquote(`$1'))'))')m4_dnl
m4_pushdef(__opt_name__,`$2')m4_dnl
m4_popdef(`__opt_name__')')

# M4OPT_CLR_OPTION(NAME)
# ----------------------
# Clears the option NAME.
m4_define(`M4OPT_CLR_OPTION',
`m4opt_list_if_member(`_m4opt_list',`$1',
`m4_popdef(m4opt_mangle_option(`$1'))m4_dnl
m4opt_list_remove(`_m4opt_list',`$1')')')

# M4OPT_ASGN_OPTION(NAME[=VAL])
# -----------------------------
# Assigns VAL to NAME.  If VAL is absent, equivalent to M4OPT_SET_OPTION(NAME).
m4_define(`M4OPT_ASGN_OPTION',
`M4OPT_SET_OPTION(m4_ifelse(m4_index(`$1',`='),-1,``$1'',`m4_patsubst(`$1',\([^=]*\)=\(.*\),``\1',`\2'')'))')

# M4OPT_UNASGN_OPTION(NAME[=VAL])
# -------------------------------
# Unassing option NAME.  Optional VAL is ignored.
m4_define(`M4OPT_UNASGN_OPTION',
`M4OPT_CLR_OPTION(m4_patsubst(`$1',\([^=]*\)=\(.*\),``\1''))')

# M4OPT_IFSET(NAME,IF-SET,IF-NOT-SET)
# -----------------------------------
# If option NAME is set, expands to IF-SET, otherwise expands to IF-NOT-SET.
m4_define(`M4OPT_IFSET',
`m4opt_list_if_member(`_m4opt_list',`$1',`$2',`$3')')

# M4OPT_VAL(NAME[,DFLT])
# ----------------------
# If option NAME is defined, return its value.  Otherwise, return DFLT.
m4_define(`M4OPT_VAL',
`m4opt_list_if_member(`_m4opt_list',`$1',`m4_defn(m4opt_mangle_option(`$1'))',`$2')')

# M4OPT_SWITCH_VAL(NAME, VAL1, CASE1, VAL2, CASE2,..., OTHERWISE)
# ---------------------------------------------------------------
m4_define(`M4OPT_SWITCH_VAL',
`m4opt_switch_val(M4OPT_VAL(`$1'),m4_shift($@))')
m4_define(`m4opt_switch_val',
`m4_ifelse(`$#',2,`$2',
`$#',1,`',
`$1',`$2',`$3',`$0(`$1',m4_shift(m4_shift(m4_shift($@))))')')')

# M4OPT_SWITCH_OPT_SET(NAME1,IF-SET1,NAME2,IF-SET2,...,IF-NOT-SET)
# -----------------------------------------------------------------
# If NAME1 is set, run IF-SET1.  Otherwise, if NAME2 is set, run IF-SET2.
# Continue the process for all name-if-set pairs within `...'.  If none
# of the options is set, run IF-NOT-SET.
m4_define(`M4OPT_SWITCH_OPT_SET',
`m4_ifelse(`$4',,`M4OPT_IFSET($@)',m4_dnl
`$3',,`M4OPT_IFSET($@)',m4_dnl
`M4OPT_IFSET(`$1',`$2',`M4OPT_SWITCH_OPT_SET(m4_shift(m4_shift($@)))')')')

# M4OPT_ASGN_OPTIONS(OPTION[,OPTION...])
# ---------------------------------------
# Assign options.
# OPTIONS is a comma-separated list of option[=value] pairs.
m4_define(`M4OPT_ASGN_OPTIONS',
`m4_foreach(`_GR_Option', m4_quote_names($@),
 `M4OPT_ASGN_OPTION(m4_quote(_GR_Option))')')

# M4OPT_UNASGN_OPTIONS(OPTION[,OPTION...])
# -----------------------------------------
# Unassign options.
m4_define(`M4OPT_UNASGN_OPTIONS',
`m4_foreach(`_GR_Option', m4_quote_names($@),
 `M4OPT_UNASGN_OPTION(m4_quote(_GR_Option))')')

# M4OPT_WITH_OPTIONS(CODE,OPT...)
# --------------------------------
# Expand to CODE with the options from OPTS... set.
m4_define(`M4OPT_WITH_OPTIONS',
`M4OPT_PUSH_OPTIONS(m4_shift($@))m4_dnl
$1
M4OPT_POP_OPTIONS(m4_shift($@))')

# M4OPT_PUSH_OPTIONS()
# --------------------
m4_define(`M4OPT_PUSH_OPTIONS',
`m4_pushdef(`_m4opt_list')m4_dnl
M4OPT_ASGN_OPTIONS($@)')

# M4OPT_POP_OPTIONS()
# -------------------
m4_define(`M4OPT_POP_OPTIONS',
`M4OPT_FOREACH(`opt',`m4_popdef(m4opt_mangle_option(opt))')m4_dnl
m4_popdef(`_m4opt_list')')

# M4OPT_FOREACH(ITERATOR, TEXT)
# -----------------------------
# Takes the name in ITERATOR, which must be a valid option name, and
# successively assigns it each option name, in the same order they were
# created.  For each assignment to ITERATOR, TEXT is appended to the expansion
# of M4OPT_FOREACH.  TEXT may refer to ITERATOR.  Any definition of ITERATOR
# prior to this invocation is restored.
#
m4_define(`M4OPT_FOREACH',
  `m4_foreach(`$1',m4_quote_names(_m4opt_list),`$2')')

# M4OPT_DUMP([PREFIX])
# --------------------
# Dump all option definitions to stdout, each on a separate line, prefixed
# with PREFIX.
#
m4_define(`M4OPT_DUMP',
`m4_pushdef(`__dump_newline__')m4_dnl
M4OPT_FOREACH(`opt',`m4_ifelse(__dump_newline__,,,`
')m4_define(`__dump_newline__',1)m4_dnl
`$1'opt=M4OPT_VAL(opt)')m4_dnl
m4_popdef(`__dump_newline__')')
